@@ -491,39 +491,37 @@ class Chemical:
     ### Creators ###
     
     def __new__(cls, ID, *, search_ID=None,
-                eos=PR, phase_ref=None,
-                default=False, phase=None,
-                CAS=None, search_db=True,
+                eos=PR, phase_ref=None, CAS=None,
+                default=False, phase=None, search_db=True, 
                 V=None, Cn=None, mu=None, Cp=None, rho=None,
                 sigma=None, kappa=None, epsilon=None,
                 **data):
         cache = cls._cache
-        search_ID = search_ID or CAS or ID
-        try:
-            CAS = CAS_from_any(search_ID)
-        except Exception as Error:
-            search_ID = to_searchable_format(search_ID)
-            try: CAS = CAS_from_any(search_ID)
-            except:
-                if search_db: raise Error
-        if search_db and CAS in cache:
-            self = cache[CAS]
-            if ID != self._ID:
-                if ID in cache:
-                    self = cache[ID]
-                else:
-                    cache[ID] = self = self.copy(ID=ID, CAS=self._CAS)
-            setfield = setattr
-            for i,j in data.items(): 
-                if j is not None: setfield(self, i, j)
-        else:
-            if CAS:
+        search_ID = search_ID or ID
+        if search_db:
+            try:
+                CAS = CAS_from_any(search_ID)
+            except Exception as Error:
+                search_ID = to_searchable_format(search_ID)
+                try: CAS = CAS_from_any(search_ID)
+                except: raise Error
+            if CAS in cache:
+                self = cache[CAS]
+                if ID != self._ID:
+                    if ID in cache:
+                        self = cache[ID]
+                    else:
+                        cache[ID] = self = self.copy(ID=ID, CAS=self._CAS)
+                setfield = setattr
+                for i,j in data.items(): 
+                    if j is not None: setfield(self, i, j)
+            else:
                 self = super().__new__(cls)
                 self._ID = ID
                 self.reset(CAS, eos, phase_ref, **data)
-            else:
-                self = cls.blank(ID, CAS, phase_ref, **data)                
-            cache[CAS] = self
+                cache[CAS] = self
+        else:
+            self = cls.blank(ID, CAS, phase_ref, **data)
         if phase: self = self.at_state(phase, copy=True)
         if mu: self.mu.add_model(mu, top_priority=True)
         if Cn: self.Cn.add_model(Cn, top_priority=True)
