@@ -6,6 +6,7 @@ Created on Sat Nov 30 21:40:59 2019
 """
 import flexsolve as flx
 from ..functional import normalize
+from .fugacity_coefficients import IdealFugacityCoefficients
 import numpy as np
 
 __all__ = ('solve_x', 'solve_y')
@@ -30,7 +31,8 @@ def solve_x(x_gamma_poyinting, gamma, poyinting, T, x_guess):
     try:
         x = flx.aitken(x_iter, x_guess, 1e-8, args=args)
     except flx.SolverError as solver_error:
-        x = flx.fixed_point(x_iter, solver_error.x, 1e-8, args=args)
+        try: x = flx.fixed_point(x_iter, solver_error.x, 1e-8, args=args)
+        except: x = x_gamma_poyinting
     except flx.InfeasibleRegion:
         x = x_gamma_poyinting
     return x
@@ -40,5 +42,6 @@ def y_iter(y, y_phi, phi, T, P):
     return y_phi / phi(y, T, P)
 
 def solve_y(y_phi, phi, T, P, y_guess):
+    if isinstance(phi, IdealFugacityCoefficients): return y_phi
     if y_guess is None: y_guess = y_phi
     return flx.aitken(y_iter, y_phi, 1e-8, args=(y_phi, phi, T, P))
\ No newline at end of file
