@@ -24,19 +24,18 @@ References
        (June 1997): 11-34. doi:10.1016/S0378-3812(97)00058-7.
 '''
 
-from __future__ import division
-
 __all__ = ['BVirial_Pitzer_Curl', 'BVirial_Abbott', 'BVirial_Tsonopoulos',
            'BVirial_Tsonopoulos_extended']
 
 from scipy.misc import derivative
 from math import log
+from flexsolve import njitable
 from .._constants import R
 
 
 ### Second Virial Coefficients
 
-
+@njitable
 def BVirial_Pitzer_Curl(T, Tc, Pc, omega, order=0):
     r'''Calculates the second virial coefficient using the model in [1]_.
     Designed for simple calculations.
@@ -62,8 +61,8 @@ def BVirial_Pitzer_Curl(T, Tc, Pc, omega, order=0):
         Order of the calculation. 0 for the calculation of B itself; for 1/2/3, 
         the first/second/third derivative of B with respect to temperature; and  
         for -1/-2, the first/second indefinite integral of B with respect to 
-        temperature. No other integrals or derivatives are implemented, and an 
-        exception will be raised if any other order is given.
+        temperature. No other integrals or derivatives are implemented, and a
+        ValueError is raised if any other order is given.
 
     Returns
     -------
@@ -139,11 +138,11 @@ def BVirial_Pitzer_Curl(T, Tc, Pc, omega, order=0):
         B0 = 289*T**2/4000 - 33*T*Tc*log(T)/100 + 33*T*Tc/100 + 277*Tc**2*log(T)/2000 - 121*Tc**3/(20000*T)
         B1 = 73*T**2/2000 + 23*T*Tc*log(T)/50 - 23*T*Tc/50 + Tc**2*log(T)/2 - (20370*T**5*Tc**3 + 73*Tc**8)/(420000*T**6)
     else: 
-        raise Exception('Only orders -2, -1, 0, 1, 2 and 3 are supported.')
+        raise ValueError('only orders -2, -1, 0, 1, 2 and 3 are supported')
     Br = B0 + omega*B1
     return Br*R*Tc/Pc
 
-
+@njitable
 def BVirial_Abbott(T, Tc, Pc, omega, order=0):
     r'''Calculates the second virial coefficient using the model in [2]_.
     Simple fit to the Lee-Kesler equation.
@@ -169,8 +168,8 @@ def BVirial_Abbott(T, Tc, Pc, omega, order=0):
         Order of the calculation. 0 for the calculation of B itself; for 1/2/3, 
         the first/second/third derivative of B with respect to temperature; and  
         for -1/-2, the first/second indefinite integral of B with respect to 
-        temperature. No other integrals or derivatives are implemented, and an 
-        exception will be raised if any other order is given.
+        temperature.  No other integrals or derivatives are implemented, and a
+        ValueError is raised if any other order is given.
 
     Returns
     -------
@@ -246,11 +245,11 @@ def BVirial_Abbott(T, Tc, Pc, omega, order=0):
         B0 = 0.0415*T**2 + 211/120.*Tc**2*Tr**0.4
         B1 = 0.0695*T**2 - 43/1760.*Tc**2*Tr**(-2.2)
     else: 
-        raise Exception('Only orders -2, -1, 0, 1, 2 and 3 are supported.')
+        raise ValueError('only orders -2, -1, 0, 1, 2 and 3 are supported')
     Br = B0 + omega*B1
     return Br*R*Tc/Pc
 
-
+@njitable
 def BVirial_Tsonopoulos(T, Tc, Pc, omega, order=0):
     r'''Calculates the second virial coefficient using the model in [3]_.
 
@@ -275,8 +274,8 @@ def BVirial_Tsonopoulos(T, Tc, Pc, omega, order=0):
         Order of the calculation. 0 for the calculation of B itself; for 1/2/3, 
         the first/second/third derivative of B with respect to temperature; and  
         for -1/-2, the first/second indefinite integral of B with respect to 
-        temperature. No other integrals or derivatives are implemented, and an 
-        exception will be raised if any other order is given.
+        temperature. No other integrals or derivatives are implemented, and a
+        ValueError is raised if any other order is given.
 
     Returns
     -------
@@ -355,10 +354,46 @@ def BVirial_Tsonopoulos(T, Tc, Pc, omega, order=0):
         B0 = 289*T**2/4000. - 33*T*Tc*log(T)/100. + 33*T*Tc/100. + 277*Tc**2*log(T)/2000. - (254100*T**5*Tc**3 + 607*Tc**8)/(42000000.*T**6)
         B1 = 637*T**2/20000. - 331*Tc**2*log(T)/1000. - (44415*T**5*Tc**3 + 40*Tc**8)/(210000.*T**6)
     else: 
-        raise Exception('Only orders -2, -1, 0, 1, 2 and 3 are supported.')
+        raise ValueError('only orders -2, -1, 0, 1, 2 and 3 are supported')
     Br = (B0+omega*B1)
     return Br*R*Tc/Pc
 
+@njitable
+def coefficients_Tsonopoulos_extended(T, Tc, order):
+    Tr = T/Tc
+    if order == 0:
+        B0 = 0.1445 - 0.33/Tr - 0.1385/Tr**2 - 0.0121/Tr**3 - 0.000607/Tr**8
+        B1 = 0.0637 + 0.331/Tr**2 - 0.423/Tr**3 - 0.008/Tr**8
+        B2 = 1./Tr**6
+        B3 = -1./Tr**8
+    elif order == 1:
+        B0 = 33*Tc/(100*T**2) + 277*Tc**2/(1000*T**3) + 363*Tc**3/(10000*T**4) + 607*Tc**8/(125000*T**9)
+        B1 = -331*Tc**2/(500*T**3) + 1269*Tc**3/(1000*T**4) + 8*Tc**8/(125*T**9)
+        B2 = -6.0*Tc**6/T**7
+        B3 = 8.0*Tc**8/T**9
+    elif order == 2:
+        B0 = -3*Tc*(27500 + 34625*Tc/T + 6050*Tc**2/T**2 + 1821*Tc**7/T**7)/(125000*T**3)
+        B1 = 3*Tc**2*(331 - 846*Tc/T - 96*Tc**6/T**6)/(500*T**4)
+        B2 = 42.0*Tc**6/T**8
+        B3 = -72.0*Tc**8/T**10
+    elif order == 3:
+        B0 = 3*Tc*(8250 + 13850*Tc/T + 3025*Tc**2/T**2 + 1821*Tc**7/T**7)/(12500*T**4)
+        B1 = 3*Tc**2*(-662 + 2115*Tc/T + 480*Tc**6/T**6)/(250*T**5)
+        B2 = -336.0*Tc**6/T**9
+        B3 = 720.0*Tc**8/T**11
+    elif order == -1:
+        B0 = 289*T/2000. - 33*Tc*log(T)/100. + (969500*T**6*Tc**2 + 42350*T**5*Tc**3 + 607*Tc**8)/(7000000.*T**7)
+        B1 = 637*T/10000. - (23170*T**6*Tc**2 - 14805*T**5*Tc**3 - 80*Tc**8)/(70000.*T**7)
+        B2 = -Tc**6/(5*T**5)
+        B3 = Tc**8/(7*T**7)
+    elif order == -2:
+        B0 = 289*T**2/4000. - 33*T*Tc*log(T)/100. + 33*T*Tc/100. + 277*Tc**2*log(T)/2000. - (254100*T**5*Tc**3 + 607*Tc**8)/(42000000.*T**6)
+        B1 = 637*T**2/20000. - 331*Tc**2*log(T)/1000. - (44415*T**5*Tc**3 + 40*Tc**8)/(210000.*T**6)
+        B2 = Tc**6/(20*T**4)
+        B3 = -Tc**8/(42*T**6)
+    else: 
+        raise ValueError('only orders -2, -1, 0, 1, 2 and 3 are supported')
+    return B0, B1, B2, B3
 
 def BVirial_Tsonopoulos_extended(T, Tc, Pc, omega, a, b, species_type, dipole, order):
     r'''Calculates the second virial coefficient using the
@@ -393,15 +428,15 @@ def BVirial_Tsonopoulos_extended(T, Tc, Pc, omega, a, b, species_type, dipole, o
         Fit parameter, calculated based on species_type if a is not given and
         species_type matches on of the supported chemical classes.
     species_type : str, optional
-        One of .
+        See notes.
     dipole : float
         dipole moment, optional, [Debye]
     order : int, optional
         Order of the calculation. 0 for the calculation of B itself; for 1/2/3, 
         the first/second/third derivative of B with respect to temperature; and  
         for -1/-2, the first/second indefinite integral of B with respect to 
-        temperature. No other integrals or derivatives are implemented, and an 
-        exception will be raised if any other order is given.
+        temperature. No other integrals or derivatives are implemented, and a
+        ValueError is raised if any other order is given.
 
     Returns
     -------
@@ -531,41 +566,9 @@ def BVirial_Tsonopoulos_extended(T, Tc, Pc, omega, a, b, species_type, dipole, o
     -9.679715056695323e-05
 
     '''
-    Tr = T/Tc
-    if order == 0:
-        B0 = 0.1445 - 0.33/Tr - 0.1385/Tr**2 - 0.0121/Tr**3 - 0.000607/Tr**8
-        B1 = 0.0637 + 0.331/Tr**2 - 0.423/Tr**3 - 0.008/Tr**8
-        B2 = 1./Tr**6
-        B3 = -1./Tr**8
-    elif order == 1:
-        B0 = 33*Tc/(100*T**2) + 277*Tc**2/(1000*T**3) + 363*Tc**3/(10000*T**4) + 607*Tc**8/(125000*T**9)
-        B1 = -331*Tc**2/(500*T**3) + 1269*Tc**3/(1000*T**4) + 8*Tc**8/(125*T**9)
-        B2 = -6.0*Tc**6/T**7
-        B3 = 8.0*Tc**8/T**9
-    elif order == 2:
-        B0 = -3*Tc*(27500 + 34625*Tc/T + 6050*Tc**2/T**2 + 1821*Tc**7/T**7)/(125000*T**3)
-        B1 = 3*Tc**2*(331 - 846*Tc/T - 96*Tc**6/T**6)/(500*T**4)
-        B2 = 42.0*Tc**6/T**8
-        B3 = -72.0*Tc**8/T**10
-    elif order == 3:
-        B0 = 3*Tc*(8250 + 13850*Tc/T + 3025*Tc**2/T**2 + 1821*Tc**7/T**7)/(12500*T**4)
-        B1 = 3*Tc**2*(-662 + 2115*Tc/T + 480*Tc**6/T**6)/(250*T**5)
-        B2 = -336.0*Tc**6/T**9
-        B3 = 720.0*Tc**8/T**11
-    elif order == -1:
-        B0 = 289*T/2000. - 33*Tc*log(T)/100. + (969500*T**6*Tc**2 + 42350*T**5*Tc**3 + 607*Tc**8)/(7000000.*T**7)
-        B1 = 637*T/10000. - (23170*T**6*Tc**2 - 14805*T**5*Tc**3 - 80*Tc**8)/(70000.*T**7)
-        B2 = -Tc**6/(5*T**5)
-        B3 = Tc**8/(7*T**7)
-    elif order == -2:
-        B0 = 289*T**2/4000. - 33*T*Tc*log(T)/100. + 33*T*Tc/100. + 277*Tc**2*log(T)/2000. - (254100*T**5*Tc**3 + 607*Tc**8)/(42000000.*T**6)
-        B1 = 637*T**2/20000. - 331*Tc**2*log(T)/1000. - (44415*T**5*Tc**3 + 40*Tc**8)/(210000.*T**6)
-        B2 = Tc**6/(20*T**4)
-        B3 = -Tc**8/(42*T**6)
-    else: 
-        raise Exception('Only orders -2, -1, 0, 1, 2 and 3 are supported.')
+    B0, B1, B2, B3 = coefficients_Tsonopoulos_extended(T, Tc, order)
     if a == 0 and b == 0 and species_type != '':
-        if species_type == 'simple' or species_type == 'normal':
+        if species_type in {'simple', 'normal'}:
             a, b = 0, 0
         elif species_type == 'methyl alcohol':
             a, b = 0.0878, 0.0525
@@ -573,13 +576,10 @@ def BVirial_Tsonopoulos_extended(T, Tc, Pc, omega, a, b, species_type, dipole, o
             a, b = -0.0109, 0
         elif dipole != 0 and Tc != 0 and Pc != 0:
             dipole_r = 1E5*dipole**2*(Pc/101325.0)/Tc**2
-
-            if (species_type == 'ketone' or species_type == 'aldehyde'
-            or species_type == 'alkyl nitrile' or species_type == 'ether'
-            or species_type == 'carboxylic acid' or species_type == 'ester'):
+            if species_type in {'ketone', 'aldehyde', 'alkyl nitrile', 'ether',
+                                'carboxylic acid', 'ester'}:
                 a, b = -2.14E-4*dipole_r-4.308E-21*dipole_r**8, 0
-            elif (species_type == 'alkyl halide' or species_type == 'mercaptan'
-            or species_type == 'sulfide' or species_type == 'disulfide'):
+            elif species_type in {'alkyl halide', 'mercaptan', 'sulfide', 'disulfide'}:
                 a, b = -2.188E-4*dipole_r**4-7.831E-21*dipole_r**8, 0
 
             elif species_type == 'alkanol':
